<?php

namespace Drupal\Tests\footnotes\Functional;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Tests\block\Functional\AssertBlockAppearsTrait;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\ckeditor5\Traits\CKEditor5TestTrait;
use Drupal\Tests\footnotes\Traits\FootnotesTestHelperTrait;

/**
 * Test the footnotes group block.
 *
 * @group footnotes
 */
class FootnotesGroupBlockTest extends BrowserTestBase {

  use StringTranslationTrait;
  use CKEditor5TestTrait;
  use FootnotesTestHelperTrait;
  use AssertBlockAppearsTrait;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'ckeditor5',
    'filter',
    'ckeditor5_test',
    'footnotes',
    'block',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->setUpFormatAndEditor();

    $this->updateEditorSettings([
      'footnotes_collapse' => FALSE,
      'footnotes_css' => FALSE,
      'footnotes_footer_disable' => TRUE,
    ]);

    $this->setUpNodeTypeAndField();
    $this->setUpDefaultUserWithAccess();
  }

  /**
   * Tests CKEditor5 plugin dialog loads.
   */
  public function testGroupBlock() {

    // Build some default footnotes.
    $note1_build = $this->buildFootnote('Note <b>one</b> test');
    $note1 = $this->renderFootnote($note1_build);
    $note2_build = $this->buildFootnote('Note <b>two</b> test');
    $note2 = $this->renderFootnote($note2_build);
    $body = '<p>testFootnotesFooterDisable' . $this->randomMachineName(100) . $note1 . '</p>';
    $body .= '<p>' . $this->randomMachineName(100) . $note2 . '</p>';

    // Add the footnotes group block.
    $settings = [
      'theme' => 'stark',
      'region' => 'footer',
      'label' => 'Footnotes Group',
      'context_mapping' => [
        'entity' => '@node.node_route_context:node',
      ],
      'group_via_js' => FALSE,
    ];

    $this->drupalPlaceBlock('footnotes_group', $settings);

    // Create a node.
    $node = $this->drupalCreateNode([
      'title' => $this->randomString(),
      'type' => 'page',
      'body' => [
        'value' => $body,
        'format' => 'filtered_html',
      ],
    ]);

    $this->drupalGet('node/' . $node->id());
    $this->assertSession()->elementExists('css', '#footnotes_group');
    $this->assertSession()->elementTextContains('css', '#footnotes_group li:first-child', 'Note one test');
    $this->assertSession()->elementTextContains('css', '#footnotes_group li:last-child', 'Note two test');

    // Make another request to ensure the block is visible with caching.
    $this->drupalGet('node/' . $node->id());
    $this->assertSession()->elementExists('css', '#footnotes_group');
    $this->assertSession()->elementTextContains('css', '#footnotes_group li:first-child', 'Note one test');
    $this->assertSession()->elementTextContains('css', '#footnotes_group li:last-child', 'Note two test');

    // Assert X-Drupal-Dynamic-Cache reports as HIT.
    $this->assertSession()->responseHeaderEquals('X-Drupal-Dynamic-Cache', 'HIT');
  }

}
